/*
        LCD Display Driver

        Copyrights 2009-2010, Yoshi Hiroshi
*/

#include <avr/io.h>
#include <util/delay.h>
#include "lcd.h"

unsigned char buff[2][20];
unsigned char LCD_Width;
unsigned char LCD_Lines;

//////////////////////////////////////////////////////
void LCD_Init(void)
{
	LCD_Width = 8;
	LCD_Lines = 2;
	LCD_DATA_DDR |= LCD_DATA_MASK;
	LCD_CTRL_DDR |= LCD_CTRL_MASK;
	LCD_CTRL_PORT &= ~LCD_CTRL_MASK;
 	_delay_ms(100);
	LCD_SendCommandNibble(LCD_COMMAND_8BITMODE);
	_delay_ms(4);
	LCD_SendCommandNibble(LCD_COMMAND_8BITMODE);
	_delay_ms(4);
	LCD_SendCommandNibble(LCD_COMMAND_8BITMODE);
	LCD_SendCommandNibble(LCD_COMMAND_4BITMODE);
	LCD_SendCommand(LCD_COMMAND_FUNCTIONSET);
	LCD_SendCommand(LCD_COMMAND_DISP_CONTROL | 
					LCD_COMMAND_DISPCTRL_DISP_OFF );
	LCD_Clear();
	LCD_SendCommand(LCD_COMMAND_ENTRYMODE | 
					LCD_COMMAND_ENTRYMODE_INC);
	LCD_SendCommand(LCD_COMMAND_DISP_CONTROL | 
					LCD_COMMAND_DISPCTRL_DISP_ON |
					LCD_COMMAND_DISPCTRL_CRSL_ON );
}

//////////////////////////////////////////////////////
void LCD_SendCommandNibble(unsigned char nibbleCommand)
{
	LCD_CTRL_PORT &= ~LCD_CTRL_MASK;                	// Clear RS, RW, E
	LCD_DATA_PORT &= ~LCD_DATA_MASK;
	LCD_DATA_PORT |= (nibbleCommand & 0x0F) << LCD_DATA_POS;
	LCD_CTRL_PORT |= _BV(LCD_CTRL_ENABLE);				// Set   E
    _delay_us(5);
	LCD_CTRL_PORT &= ~_BV(LCD_CTRL_ENABLE);
}

//////////////////////////////////////////////////////
void LCD_SendCommand(unsigned char command)
{
	_delay_us(40);
	while(LCD_IsBusy());
	LCD_SendCommandNibble(command >> 4);
	LCD_SendCommandNibble(command);
}

//////////////////////////////////////////////////////
void LCD_SendDataNibble(unsigned char nibbleData)
{
	LCD_CTRL_PORT &= ~LCD_CTRL_MASK;                	// Clear RW, RS, E
	LCD_DATA_PORT &= ~LCD_DATA_MASK;
	LCD_DATA_PORT |= (nibbleData & 0x0F) << LCD_DATA_POS;
	LCD_CTRL_PORT |= _BV(LCD_CTRL_RS);					// Set   RS
	LCD_CTRL_PORT |= _BV(LCD_CTRL_ENABLE);				// Set   E
    _delay_us(5);
	LCD_CTRL_PORT &= ~_BV(LCD_CTRL_ENABLE);
}

//////////////////////////////////////////////////////
void LCD_SendData(unsigned char data)
{
	_delay_us(40);
	while(LCD_IsBusy());
	LCD_SendDataNibble(data >> 4);
	LCD_SendDataNibble(data);
}

//////////////////////////////////////////////////////
void LCD_SendString(unsigned char *str)
{
	while(*str != 0)
	{
		LCD_SendData(*str++);
	}
}

//////////////////////////////////////////////////////
unsigned char LCD_IsBusy(void)
{
	unsigned char busyFlag;
	LCD_CTRL_PORT &= ~LCD_CTRL_MASK;
	LCD_DATA_DDR  &= ~LCD_DATA_MASK;
	LCD_DATA_PORT |= LCD_DATA_MASK;
	LCD_CTRL_PORT |= _BV(LCD_CTRL_RW);
	LCD_CTRL_PORT |= _BV(LCD_CTRL_ENABLE);				// Set   E
    _delay_us(5);
	busyFlag = (LCD_DATA_PIN & _BV(LCD_BUSY_FLAG));
	LCD_CTRL_PORT &= ~_BV(LCD_CTRL_ENABLE);
	LCD_DATA_DDR |= LCD_DATA_MASK;
	return(busyFlag != 0);
//  return 0;
}

//////////////////////////////////////////////////////
void LCD_Clear(void)
{
	LCD_SendCommand(LCD_COMMAND_CLEAR);
	for(unsigned char index = 0; index < LCD_Lines; index++)
	{
		for(unsigned char count = 0; count < LCD_Width; count++)
		{
			buff[index][count] = ' ';
		}
	}
    _delay_ms(10);
}

//////////////////////////////////////////////////////
void LCD_Home(void)
{
	LCD_SendCommand(LCD_COMMAND_HOME);
	_delay_ms(5);
}

//////////////////////////////////////////////////////
void LCD_SetPosition(unsigned char line, unsigned char column)
{
	if(line > LCD_Lines) line = 0;
	if(column > LCD_Width) column = 0;
	LCD_SendCommand(LCD_COMMAND_SET_ADDRESS + 
							((line % 2) * 0x40) + 
							column + 
							((line / 2) * LCD_Width));
}

//////////////////////////////////////////////////////
void LCD_Print(char* str)
{
	while(*str != 0)
	{
		LCD_SendData(*str++);
	}
}

