/*
        LCD Display Driver Header File

        Copyrights 2009-2010, Yoshi Hiroshi
*/

#ifndef __LCD_HEADER_INCLUDED__
#define __LCD_HEADER_INCLUDED__

#if defined (__AVR_ATtiny84__)
//
/*
	PB0 R/W - lcd
	PB1 E   - lcd
	PB2 RS  - lcd
	PA4 DB4 - lcd
	PA5 DB5 - lcd
	PA6 DB6 - lcd
	PA7 DB7 - lcd
*/
//

//---------------------------------------------------------------------
#define	LCD_CTRL_RS		PB2			    // Register Select 1:Data 0:Command
#define	LCD_CTRL_ENABLE	PB1			    // ENABLE Signal 1:Enable 0:Disable
#define	LCD_CTRL_RW		PB0			    // RW Signal 	   1:Read 0:Write
#define LCD_CTRL_PORT	PORTB		    // LCD Control Port
#define LCD_CTRL_DDR	DDRB		    // LCD Control Port Data Direction Register
#define LCD_CTRL_MASK	(_BV(LCD_CTRL_RS)|_BV(LCD_CTRL_ENABLE)|_BV(LCD_CTRL_RW))

//---------------------------------------------------------------------
#define LCD_DATA_PORT	PORTA		    // LCD Data Port
#define LCD_DATA_PIN	PINA		    // LCD Data Input Port
#define LCD_DATA_POS	4		    	// LCD Data Position
#define LCD_DATA_DDR 	DDRA	    	// LCD Data Port Data Direction Register
#define LCD_DATA_MASK	(0xF0)
#define LCD_BUSY_FLAG	PA7

#elif defined (__AVR_ATtiny861__)
//
/*
	PB0 R/W - lcd
	PB1 E   - lcd
	PB2 RS  - lcd
	PB3 DB4 - lcd
	PB4 DB5 - lcd
	PB5 DB6 - lcd
	PB6 DB7 - lcd
*/
//

//---------------------------------------------------------------------
#define	LCD_CTRL_RS		PORTB2			// Register Select 1:Data 0:Command
#define	LCD_CTRL_ENABLE	PORTB1			// ENABLE Signal 1:Enable 0:Disable
#define	LCD_CTRL_RW		PORTB0			// RW Signal 	   1:Read 0:Write
#define LCD_CTRL_PORT	PORTB		    // LCD Control Port
#define LCD_CTRL_DDR	DDRB		    // LCD Control Port Data Direction Register
#define LCD_CTRL_MASK	(_BV(LCD_CTRL_RS)|_BV(LCD_CTRL_ENABLE)|_BV(LCD_CTRL_RW))

//---------------------------------------------------------------------
#define LCD_DATA_PORT	PORTB		    // LCD Data Port
#define LCD_DATA_PIN	PINB		    // LCD Data Input Port
#define LCD_DATA_POS	3			    // LCD Data Position
#define LCD_DATA_DDR 	DDRB		    // LCD Data Port Data Direction Register
#define LCD_DATA_MASK	(0x78)
#define LCD_BUSY_FLAG	PORTB6

#elif defined (__AVR_ATmega8__) || (__AVR_ATmega88P__) || defined (__AVR_ATmega168__) || defined (__AVR_ATmega168P__)
//
/*
	PD0 R/W - lcd
	PD1 E   - lcd
	PD2 RS  - lcd
	PD4 DB4 - lcd
	PD5 DB5 - lcd
	PD6 DB6 - lcd
	PD7 DB7 - lcd
*/
//

//---------------------------------------------------------------------
#define	LCD_CTRL_RS		PORTD2			// Register Select 1:Data 0:Command
#define	LCD_CTRL_ENABLE	PORTD1			// ENABLE Signal 1:Enable 0:Disable
#define	LCD_CTRL_RW		PORTD0			// RW Signal 	   1:Read 0:Write
#define LCD_CTRL_PORT	PORTD		    // LCD Control Port
#define LCD_CTRL_DDR	DDRD		    // LCD Control Port Data Direction Register
#define LCD_CTRL_MASK	(_BV(LCD_CTRL_RS)|_BV(LCD_CTRL_ENABLE)|_BV(LCD_CTRL_RW))

//---------------------------------------------------------------------
#define LCD_DATA_PORT	PORTD		    // LCD Data Port
#define LCD_DATA_PIN	PIND		    // LCD Data Input Port
#define LCD_DATA_POS	4			    // LCD Data Position
#define LCD_DATA_DDR 	DDRD		    // LCD Data Port Data Direction Register
#define LCD_DATA_MASK	(0xF0)
#define LCD_BUSY_FLAG	PORTD7

#endif

//---------------------------------------------------------------------



#define LCD_COMMAND_8BITMODE	        	0x03
#define LCD_COMMAND_4BITMODE	        	0x02
#define LCD_COMMAND_FUNCTIONSET	        	0x28
#define LCD_COMMAND_CLEAR		        	0x01
#define LCD_COMMAND_HOME		        	0x02
#define LCD_COMMAND_ENTRYMODE	        	0x04
#define 	LCD_COMMAND_ENTRYMODE_INC	    	0x02
#define 	LCD_COMMAND_ENTRYMODE_DEC	    	0x00
#define 	LCD_COMMAND_ENTRYMODE_SHIFT	    	0x01
#define 	LCD_COMMAND_ENTRYMODE_NOSHIFT   	0x00
#define LCD_COMMAND_DISP_CONTROL        	0x08
#define 	LCD_COMMAND_DISPCTRL_DISP_ON    	0x04
#define 	LCD_COMMAND_DISPCTRL_DISP_OFF   	0x00
#define 	LCD_COMMAND_DISPCTRL_CRSL_ON    	0x02
#define 	LCD_COMMAND_DISPCTRL_CRSL_OFF   	0x00
#define 	LCD_COMMAND_DISPCTRL_BLNK_ON    	0x01
#define 	LCD_COMMAND_DISPCTRL_BLNK_OFF   	0x00
#define LCD_COMMAND_SET_ADDRESS	        	0x80

void LCD_SendCommandNibble(unsigned char nibbleCommand);
void LCD_SendCommand(unsigned char command);
void LCD_SendDataNibble(unsigned char nibbleData);
void LCD_SendData(unsigned char data);
void LCD_SendString(unsigned char *str);
unsigned char  LCD_IsBusy(void);

void LCD_Init(void);
void LCD_Clear(void);
void LCD_Home(void);
void LCD_SetPosition(unsigned char x, unsigned char y);
void LCD_Print(char* str);

#endif
